#ifdef GLES
precision mediump float;
#endif
uniform mat4 u_mvp;
uniform float u_phase;
uniform float u_scale;
uniform float u_lscale;
attribute vec2 a_pos;
varying vec2 v_tex;

void main() {
    gl_Position = u_mvp * vec4(a_pos * u_scale * u_lscale * u_phase, 0.0, 1.0);
    v_tex = a_pos;
}

$$

#ifdef GLES
precision mediump float;
#endif
varying vec2 v_tex;
uniform float u_scale;
uniform vec2 u_dir;
uniform mediump int u_mode;
uniform vec4 u_color;
uniform vec4 u_scolor;
uniform mediump int u_lmode;
uniform mediump int u_rmode;
uniform float u_dscale;
uniform float u_lscale;
uniform float u_rscale;

float line_distance(vec2 p, vec2 p1, vec2 p2) {
    vec2 center = (p1 + p2) * 0.5;
    float len = length(p2 - p1);
    vec2 dir = (p2 - p1) / len;
    vec2 rel_p = p - center;
    return dot(rel_p, vec2(dir.y, -dir.x));
}

float arrow_stealth(vec2 texcoord, float body, float head, float height) {
    vec2 end = +vec2(body / 2.0, 0.0);

    float d1 = line_distance(texcoord, end - head * vec2(+1.0, -height), end);
    float d2 = line_distance(texcoord, end - head * vec2(+1.0, -height), end - vec2(3.0 * head / 4.0, 0.0));
    float d3 = line_distance(texcoord, end - head * vec2(+1.0, +height), end);
    float d4 = line_distance(texcoord, end - head * vec2(+1.0, +height), end - vec2(3.0 * head / 4.0, 0.0));

    return max(max(-d1, d3), -max(-d2, d4));
}

vec4 outline(float distance, float linewidth, float antialias, vec4 stroke, vec4 fill, vec4 outer) {
    vec4 frag_color;
    float t = linewidth / 2.0 - antialias;
    float signed_distance = distance;
    float border_distance = abs(signed_distance) - t;
    float alpha = border_distance / antialias;
    alpha = exp(-alpha * alpha);

    if (border_distance < 0.0)
        frag_color = stroke;
    else if (signed_distance < 0.0)
        frag_color = mix(fill, stroke, sqrt(alpha));
    else
        frag_color = mix(outer, stroke, sqrt(alpha));
    return frag_color;
}

void main() {
    float len = 1.0 - length(v_tex);
    if (u_mode == -1)
        gl_FragColor = u_color * len;
    else if (u_lmode == 1 && u_mode == 1) {
        // outer ring
        float a = 0.5 * smoothstep(0.0, 2.0 / u_scale / u_lscale * u_dscale, len);
        // inner ring
        float b = 0.3 * smoothstep(3.0 / u_scale / u_lscale * u_dscale, 4.0 / u_scale / u_lscale * u_dscale, len);
        // arrow
        vec2 texcoord = vec2(dot(v_tex, u_dir), cross(vec3(v_tex, 0.0), vec3(u_dir, 0.0)).z) * u_scale / u_rscale;
        float arrow;
        if (u_rmode == 1)
            arrow = arrow_stealth(texcoord, 0.0, 0.8, 0.4);
        else
            arrow = arrow_stealth(texcoord, 1.0, 1.0, 0.4);
        // - subtract inner from outer to create the outline
        // - add arrow
        gl_FragColor = outline(arrow, 0.05, 0.01, u_scolor, u_color, u_color * (a - b));
    } else {
        // outer ring
        float a = 0.5 * smoothstep(0.0, 2.0 / u_scale / u_lscale * u_dscale, len);
        // inner ring
        float b = 0.3 * smoothstep(3.0 / u_scale / u_lscale * u_dscale, 4.0 / u_scale / u_lscale * u_dscale, len);
        // center point
        float c = 1.0 - smoothstep(20.0 / u_scale * u_dscale, 22.0 / u_scale * u_dscale, 1.0 - len);
        float cs = 1.0 - smoothstep(23.0 / u_scale * u_dscale, 26.0 / u_scale * u_dscale, 1.0 - len);
        vec2 dir = normalize(v_tex);
        float d = dot(dir, u_dir);
        // 0.5 width of viewshed
        len = 1.0 - length(v_tex * u_scale / u_rscale);
        d = clamp(smoothstep(0.5, 0.5 + 2.0 / u_scale * u_dscale, d - cs) * len, 0.0, 1.0);
        // - subtract inner from outer to create the outline
        // - multiply by viewshed
        // - add center point
        a = max(d, a - b) + c;
        gl_FragColor = u_color * a + u_scolor * (cs - c);
    }
}
